#!/usr/bin/python -B

import config

# the character used to separate between the Linux version number digits
VERSION_DIGIT_SEPARATOR = '.'
	
# the charcater that separates the package version from the suffix
PACKAGE_NAME_SUFFIX_SEPARATOR = '-'

# the major version, minor version and release digits in a Linux version number
# ex.: for Linux 2.6.32.41, the major version number is 2.6.32, the minor 
# version number is 41 and the release number is 32
VERSION_DIGITS = { 'MAJOR'	:	[ 0, 1, 2 ],
				   'MINOR'	:	3,
				   'RELEASE'	:	2 }

# the PNG image used as the boot logo
LOGO_IMAGE = "/usr/share/doc/puppylogo96.png"

# the boot logo width
LOGO_WIDTH = 80

# the boot logo height
LOGO_HEIGHT = 80

if config.LINUX_LIBRE:
	# the Linux-libre scripts to download; the former depends on the latter
	LINUX_LIBRE_SCRIPTS = "deblob-${LINUX_MAJOR_VERSION} deblob-check"
	
# the Linux image and modules package name
LINUX_PACKAGE_NAME = "linux_kernel"

# the Linux API headers package name
LINUX_HEADERS_PACKAGE_NAME = "kernel_headers"

# the Linux sources package name
LINUX_SOURCES_PACKAGE_NAME = "kernel_sources"
	
# the command to run when the script ends; the default is to play the two barks
# sound from the first boot
FINISH_COMMAND = "aplay /usr/share/sounds/2barks.au"
	   
# the output directory
OUTPUT_DIR = "$BUILD_ROOT/dist"

# the directory packages go to
PACKAGES_DIR = "$OUTPUT_DIR/packages"

# the directory sources go to
SOURCES_DIR = "$OUTPUT_DIR/sources"

# the log path
LOG_PATH = "$BUILD_ROOT/build.log"

# the directory patches go to
PATCHES_DIR = "$OUTPUT_DIR/sources/patches"

# the last item in a list
def get_list_last_item(x):	return x[len(x) - 1]

# the first item in a list
def get_list_first_item(x):	return x[0]

# turn the Linux version into a list of integers
config.LINUX_VERSION = map(int, config.LINUX_VERSION.split(VERSION_DIGIT_SEPARATOR))
	
# print the script head
print '''#!/bin/sh

######################################
# Linux build script for Puppy Linux #
######################################

################################
# Linux and Aufs configuration #
################################

# the Linux version to build
LINUX_VERSION="%s"

# the file name of the Linux sources tarball
LINUX_SOURCES_TARBALL="%s"

# the Linux download mirror
LINUX_MIRROR="%s"

# the Aufs version to use
AUFS_VERSION="%s"''' % ("".join(map(lambda x: str(x) + VERSION_DIGIT_SEPARATOR, 
						config.LINUX_VERSION))[:-1], 
						config.LINUX_SOURCES_TARBALL, 
						config.LINUX_MIRROR,
						config.AUFS_VERSION)

# print the major version number parsing part
print '''
################################
# Linux version number parsing #
################################

# get the Linux major version number
LINUX_MAJOR_VERSION=`echo $LINUX_VERSION | cut -f %s-%s -d %s`''' % (1 + get_list_first_item(VERSION_DIGITS['MAJOR']), 
																	 1 + get_list_last_item(VERSION_DIGITS['MAJOR']), 
																	 VERSION_DIGIT_SEPARATOR)

# print the Linux release number parsing part
print '''
# get the Linux release number
LINUX_RELEASE_NUMBER=`echo $LINUX_VERSION | cut -f %s -d %s`''' % (1 + VERSION_DIGITS['RELEASE'], 
																   VERSION_DIGIT_SEPARATOR)
																
# print the minor version number parsing part
print '''
# get the Linux minor version number
LINUX_MINOR_VERSION=`echo $LINUX_VERSION | cut -f %s -d %s`''' % (1 + VERSION_DIGITS['MINOR'], 
																  VERSION_DIGIT_SEPARATOR)

# print the suffix parsing part
print '''
#########################
# package configuration #
#########################

# the Linux image and modules package name
LINUX_PACKAGE_NAME="%s"

# the Linux API headers package name
LINUX_HEADERS_PACKAGE_NAME="%s"

# the Linux sources package name
LINUX_SOURCES_PACKAGE_NAME="%s"

# the package name suffix
PACKAGE_NAME_SUFFIX="${LINUX_MINOR_VERSION}%s%s"''' % (LINUX_PACKAGE_NAME,
													   LINUX_HEADERS_PACKAGE_NAME,
													   LINUX_SOURCES_PACKAGE_NAME,
													   PACKAGE_NAME_SUFFIX_SEPARATOR,
													   config.PACKAGE_NAME_SUFFIX)

# add the boot logo settings
print '''
#############
# boot logo #
#############

# the PNG image used as the boot logo
LOGO_IMAGE="%s"

# the boot logo width
LOGO_WIDTH=%s

# the boot logo height
LOGO_HEIGHT=%s''' % (LOGO_IMAGE,
					 LOGO_WIDTH,
					 LOGO_HEIGHT)

# for Linux-libre builds, add the list of required scripts
if config.LINUX_LIBRE:
	print '''
#############################
# Linux-libre configuration #
#############################

# the Linux-libre scripts to use
LINUX_LIBRE_SCRIPTS="%s"

# the Linux-libre mirror to download the scripts from
LINUX_LIBRE_MIRROR="%s"''' % (LINUX_LIBRE_SCRIPTS, 
							  config.LINUX_LIBRE_MIRROR)
							  
# print the output paths
print '''
##############################
# Output paths configuration #
##############################

# the root build directory
BUILD_ROOT=`pwd`

# the output directory
OUTPUT_DIR="%s"

# the directory packages go to
PACKAGES_DIR="%s"

# the directory sources go to
SOURCES_DIR="%s"

# the directory patches go to
PATCHES_DIR="%s"

# the log path								
LOG_PATH="%s"''' % (OUTPUT_DIR,
					PACKAGES_DIR,
					SOURCES_DIR,
					PATCHES_DIR,
					LOG_PATH)
									   
print '''
# get the build date
BUILD_DATE="`date +%d%m%Y`"

####################

# delete old logs
[ -f "$LOG_PATH" ] && rm -f "$LOG_PATH"

# create directories for the output
mkdir -p "$SOURCES_DIR" "$PATCHES_DIR" "$PACKAGES_DIR"

# download the Linux sources tarball
if [ ! -f "$SOURCES_DIR/$LINUX_SOURCES_TARBALL" ]; then
	echo "Downloading the Linux sources tarball"
	wget -P "$SOURCES_DIR" $LINUX_MIRROR/$LINUX_SOURCES_TARBALL > "$LOG_PATH" 2>&1
	if [ $? -ne 0 ]
	then
		echo "Error: failed to download the Linux sources tarball."
		exit 1
	fi
fi'''

# download the Linux-libre scripts
if config.LINUX_LIBRE:
	print '''
# check whether the Linux-libre scripts are missing
download=0
for script in $LINUX_LIBRE_SCRIPTS
do
	if [ ! -f "$SOURCES_DIR/$script" ]
	then
		download=1
		break
	fi
done

# if at least one script is missing, download it
if [ $download -eq 1 ]
then
	echo "Downloading the Linux-libre deblob scripts"
	for script in $LINUX_LIBRE_SCRIPTS
	do
		if [ ! -f "$SOURCES_DIR/$script" ]
		then
			wget -P "$SOURCES_DIR" $LINUX_LIBRE_MIRROR/$script >> build.log 2>&1
			if [ $? -ne 0 ]
			then
				[ -f "$SOURCES_DIR/$script" ] && rm -f "$SOURCES_DIR/$script"
				echo "Error: failed to download the $script script."
				exit 1
			fi
		fi
	done	
fi'''

print '''
# get the full Aufs package name
AUFS_FULL_PACKAGE_NAME="aufs${AUFS_VERSION}-${LINUX_RELEASE_NUMBER}-git${BUILD_DATE}"

# download the Aufs sources through git
if [ ! -f "$SOURCES_DIR/${AUFS_FULL_PACKAGE_NAME}.tar.bz2" ]
then
	echo "Downloading the Aufs sources through git"
	git clone http://git.c3sl.ufpr.br/pub/scm/aufs/aufs2-standalone.git $AUFS_FULL_PACKAGE_NAME >> "$LOG_PATH" 2>&1
	if [ $? -ne 0 ]
	then
		echo "Error: failed to download the Aufs sources through git."
		exit 1
	fi
	cd $AUFS_FULL_PACKAGE_NAME
	git checkout origin/aufs${AUFS_VERSION}-${LINUX_RELEASE_NUMBER} >> "$LOG_PATH" 2>&1
	if [ $? -ne 0 ]
	then
		echo "Error: failed to download the Aufs sources through git."
		exit 1
	fi
	rm -rf .git
	cd ..
	echo "Creating the Aufs sources tarball"
	tar -c $AUFS_FULL_PACKAGE_NAME | bzip2 -9 > "$SOURCES_DIR/${AUFS_FULL_PACKAGE_NAME}.tar.bz2"
else
	echo "Extracting the Aufs sources tarball"
	tar xf "$SOURCES_DIR/${AUFS_FULL_PACKAGE_NAME}.tar.bz2" >> "$LOG_PATH" 2>&1
	if [ $? -ne 0 ]
	then
		echo "Error: failed to extract the Aufs sources tarball."
		exit 1
	fi
fi

# patch Aufs
echo "Patching the Aufs sources"
patch -d $AUFS_FULL_PACKAGE_NAME -p1 < aufs-allow-sfs.patch >> "$LOG_PATH" 2>&1
if [ $? -ne 0 ]
then
	echo "Error: failed to patch the Aufs sources."
	exit 1
fi
cp aufs-allow-sfs.patch "$PATCHES_DIR"

# extract the Linux sources tarball
echo "Extracting the Linux sources tarball"
tar xf "$SOURCES_DIR/$LINUX_SOURCES_TARBALL" >> "$LOG_PATH" 2>&1
if [ $? -ne 0 ]
then
	echo "Error: failed to extract the Linux sources tarball."
	exit 1
fi'''

# deblob the Linux sources
if config.LINUX_LIBRE:
	print '''
echo "Deblobing the Linux sources"
echo "  creating a clean copy of the sources"
cp -r linux-${LINUX_VERSION} linux-${LINUX_VERSION}-orig
for script in $LINUX_LIBRE_SCRIPTS
do
	chmod +x "$SOURCES_DIR/$script"
done

echo "  deblobing"
cd linux-${LINUX_VERSION}
"$SOURCES_DIR/deblob-${LINUX_MAJOR_VERSION}" >> "$LOG_PATH" 2>&1
if [ $? -ne 0 ]
then
	echo "Error: failed to deblob the Linux sources."
	exit 1
fi

echo "  creating a patch"
cd ..
diff -rupN linux-${LINUX_VERSION}-orig linux-${LINUX_VERSION} > "$PATCHES_DIR/deblob.patch"
rm -rf linux-%{LINUX_VERSION}-orig'''
	
print '''

cd linux-${LINUX_VERSION}

echo "Cleaning the Linux sources"
make clean >> "$LOG_PATH" 2>&1
make mrproper >> "$LOG_PATH" 2>&1'''
	
print '''
# get the full Linux API headers package name
LINUX_HEADERS_FULL_PACKAGE_NAME="${LINUX_HEADERS_PACKAGE_NAME}-${LINUX_MAJOR_VERSION}-${PACKAGE_NAME_SUFFIX}"

echo "Creating the Linux API headers package"
echo "  checking the headers"
make headers_check >> "$LOG_PATH" 2>&1
echo "  installing the headers"
make INSTALL_HDR_PATH=$LINUX_HEADERS_FULL_PACKAGE_NAME headers_install >> "$LOG_PATH" 2>&1
echo "  removing unneeded files"
find $LINUX_HEADERS_FULL_PACKAGE_NAME/include \( -name .install -o -name ..install.cmd \) -delete
mv $LINUX_HEADERS_FULL_PACKAGE_NAME "$PACKAGES_DIR"

# patch the Linux sources with the Aufs patches and add the file system code itself
echo "Adding Aufs to the Linux sources"
for i in kbuild base standalone; do
	patch -p1 < ../$AUFS_FULL_PACKAGE_NAME/aufs2-$i.patch >> "$LOG_PATH" 2>&1
	if [ $? -ne 0 ]
	then
		echo "Error: failed to add Aufs to the Linux sources."
		exit 1
	fi
done
cp -r ../$AUFS_FULL_PACKAGE_NAME/{fs,Documentation} .
cp -r ../$AUFS_FULL_PACKAGE_NAME/include/linux/aufs_type.h include/linux
cat ../$AUFS_FULL_PACKAGE_NAME/include/linux/Kbuild >> include/linux/Kbuild

echo "Resetting the minor version number"
cp Makefile Makefile-orig
sed -i s/'^EXTRAVERSION = .*$'/'EXTRAVERSION = '/ Makefile
echo "  creating a patch"
diff -up Makefile-orig Makefile > "$PATCHES_DIR/extra-version.patch"
rm Makefile-orig

echo "Reducing the number of consoles"
cp kernel/printk.c kernel/printk.c-orig
sed -i s/'#define MAX_CMDLINECONSOLES 8'/'#define MAX_CMDLINECONSOLES 5'/ kernel/printk.c
echo "  creating a patch"
diff -up kernel/printk.c-orig kernel/printk.c > "$PATCHES_DIR/less-consoles.patch"

echo "Reducing the verbosity level"
cp -f kernel/printk.c kernel/printk.c-orig
sed -i s/'#define DEFAULT_CONSOLE_LOGLEVEL 7 \/\* anything MORE serious than KERN_DEBUG \*\/'/'#define DEFAULT_CONSOLE_LOGLEVEL 3 \/\* anything MORE serious than KERN_ERR \*\/'/ kernel/printk.c
echo "  creating a patch"
diff -up kernel/printk.c-orig kernel/printk.c > "$PATCHES_DIR/lower-verbosity.patch"

echo "Applying patches"
for patch in ../patches/*
do
	echo "  applying `basename $patch`"
	patch -p1 < "$patch" >> "$LOG_PATH" 2>&1
	if [ $? -ne 0 ]
	then
		echo "Error: failed to apply $patch on the Linux sources."
		exit 1
	fi
	cp "$patch" "$PATCHES_DIR"
done

echo "Adding the Puppy Linux boot logo"
echo "  creating a clean copy of the sources"
cp -r drivers/video/logo drivers/video/logo-orig
echo "  applying the Puppy Linux logo patch"
patch -p1 < ../puppy-logo.patch >> "$LOG_PATH" 2>&1
if [ $? -ne 0 ]
then
	echo "Error: failed to apply the Puppy Linux logo patch on the Linux sources."
	exit 1
fi

# resize the image and make the background black	
resized_image=`mktemp -u`
pngtopnm -mix -background black "$LOGO_IMAGE" | pnmscale -width $LOGO_WIDTH -height $LOGO_HEIGHT > $resized_image
if [ $? -ne 0 ]
then
	echo "Error: failed to resize ${LOGO_IMAGE}."
	exit 1
fi

# create 224-color, 16-color and monochrome variants
echo "  creating 224-color, 16-color and monochrome variants"
cat $resized_image | ppmquant 224 | pnmnoraw > drivers/video/logo/logo_puppy_clut224.ppm
cat $resized_image | ppmquant 16 | pnmnoraw > drivers/video/logo/logo_puppy_vga16.ppm
cat $resized_image | ppmquant 2 | pnmnoraw > drivers/video/logo/logo_puppy_mono.ppm

# delete the resized image
rm -f $resized_image

cd ..
echo "  creating a patch"
diff -rupN linux-${LINUX_VERSION}/drivers/video/logo-orig linux-${LINUX_VERSION}/drivers/video/logo > "$PATCHES_DIR/puppy-logo.patch"
rm -rf linux-${LINUX_VERSION}/drivers/video/logo-orig
cd linux-${LINUX_VERSION}

echo "  cleaning up patch clutter"
find . -name '*.orig' -delete
find . -name '*.rej' -delete
find . -name '*~' -delete

echo "Building Linux"
cp ../DOTconfig .config
make bzImage modules >> "$LOG_PATH" 2>&1
if [[ ! -f arch/x86/boot/bzImage || ! -f System.map ]]; then
	echo "Error: failed to build Linux."
	exit 1
fi
cp .config "$SOURCES_DIR/DOTconfig-${LINUX_VERSION}-${BUILD_DATE}"

# get the full Linux image and modules package name
LINUX_FULL_PACKAGE_NAME="${LINUX_PACKAGE_NAME}-${LINUX_RELEASE_NUMBER}-${PACKAGE_NAME_SUFFIX}"

echo "Creating the Linux image and modules package"
echo "  installing the kernel modules"
make INSTALL_MOD_PATH=$LINUX_FULL_PACKAGE_NAME modules_install'''

# delete the firmware
if not config.INCLUDE_FIRMWARE:
	print '''echo "  removing firmware"
rm -rf $LINUX_FULL_PACKAGE_NAME/lib/firmware'''

print '''echo "  removing links to the sources"
rm -f $LINUX_FULL_PACKAGE_NAME/lib/modules/$LINUX_MAJOR_VERSION/{build,source}
ln -s $LINUX_MAJOR_VERSION $LINUX_FULL_PACKAGE_NAME/lib/modules/$LINUX_VERSION
echo "  stripping modules"
for module in `find $LINUX_FULL_PACKAGE_NAME -name *.ko -type f`
do
	strip --strip-unneeded $module >> "$LOG_PATH" 2>&1
	if [ $? -ne 0 ]
	then
		echo "Error: failed to strip the Linux modules."
		exit 1
	fi
done
echo "  adding the kernel image and System.map"
mkdir $LINUX_FULL_PACKAGE_NAME/boot
cp `find arch -name bzImage -type f` $LINUX_FULL_PACKAGE_NAME/boot/vmlinuz
cp System.map $LINUX_FULL_PACKAGE_NAME/boot
mv $LINUX_FULL_PACKAGE_NAME "$PACKAGES_DIR"

echo "Cleaning the Linux sources"
make clean >> "$LOG_PATH" 2>&1
make prepare >> "$LOG_PATH" 2>&1

cd ..

# get the full Linux sources package name
LINUX_SOURCES_FULL_PACKAGE_NAME="${LINUX_SOURCES_PACKAGE_NAME}-${LINUX_RELEASE_NUMBER}-${PACKAGE_NAME_SUFFIX}"

echo "Creating a Linux sources SFS extension"
mkdir -p $LINUX_SOURCES_FULL_PACKAGE_NAME/usr/src
mv linux-${LINUX_VERSION} $LINUX_SOURCES_FULL_PACKAGE_NAME/usr/src/linux
mkdir -p $LINUX_SOURCES_FULL_PACKAGE_NAME/lib/modules/$LINUX_MAJOR_VERSION
echo "  creating required links"
ln -s /usr/src/linux $LINUX_SOURCES_FULL_PACKAGE_NAME/lib/modules/$LINUX_MAJOR_VERSION/build
ln -s /usr/src/linux $LINUX_SOURCES_FULL_PACKAGE_NAME/lib/modules/$LINUX_MAJOR_VERSION/source
mv $LINUX_SOURCES_FULL_PACKAGE_NAME "$PACKAGES_DIR"

echo "Compressing the log"
bzip2 -9 "$LOG_PATH" 
cp "${LOG_PATH}.bz2" "$SOURCES_DIR"

echo "Compressing the patches"
for patch in "$PATCHES_DIR"/*
do
	bzip2 -9 "$patch"
done

echo "Done!"'''

# do something to mark the end of the script
print FINISH_COMMAND