#!/bin/sh

######################################
# Linux build script for Puppy Linux #
######################################

################################
# Linux and Aufs configuration #
################################

# the Linux version to build
LINUX_VERSION="2.6.32.41"

# the file name of the Linux sources tarball
LINUX_SOURCES_TARBALL="linux-2.6.32.41.tar.bz2"

# the Linux download mirror
LINUX_MIRROR="http://www.il.kernel.org/pub/linux/kernel/v2.6/longterm/v2.6.32"

# the Aufs version to use
AUFS_VERSION="2.1"

################################
# Linux version number parsing #
################################

# get the Linux major version number
LINUX_MAJOR_VERSION=`echo $LINUX_VERSION | cut -f 1-3 -d .`

# get the Linux release number
LINUX_RELEASE_NUMBER=`echo $LINUX_VERSION | cut -f 3 -d .`

# get the Linux minor version number
LINUX_MINOR_VERSION=`echo $LINUX_VERSION | cut -f 4 -d .`

#########################
# package configuration #
#########################

# the Linux image and modules package name
LINUX_PACKAGE_NAME="linux_kernel"

# the Linux API headers package name
LINUX_HEADERS_PACKAGE_NAME="kernel_headers"

# the Linux sources package name
LINUX_SOURCES_PACKAGE_NAME="kernel_sources"

# the package name suffix
PACKAGE_NAME_SUFFIX="${LINUX_MINOR_VERSION}-lts-bfs"

#############
# boot logo #
#############

# the PNG image used as the boot logo
LOGO_IMAGE="/usr/share/doc/puppylogo96.png"

# the boot logo width
LOGO_WIDTH=80

# the boot logo height
LOGO_HEIGHT=80

##############################
# Output paths configuration #
##############################

# the root build directory
BUILD_ROOT=`pwd`

# the output directory
OUTPUT_DIR="$BUILD_ROOT/dist"

# the directory packages go to
PACKAGES_DIR="$OUTPUT_DIR/packages"

# the directory sources go to
SOURCES_DIR="$OUTPUT_DIR/sources"

# the directory patches go to
PATCHES_DIR="$OUTPUT_DIR/sources/patches"

# the log path								
LOG_PATH="$BUILD_ROOT/build.log"

# get the build date
BUILD_DATE="`date +%d%m%Y`"

####################

# delete old logs
[ -f "$LOG_PATH" ] && rm -f "$LOG_PATH"

# create directories for the output
mkdir -p "$SOURCES_DIR" "$PATCHES_DIR" "$PACKAGES_DIR"

# download the Linux sources tarball
if [ ! -f "$SOURCES_DIR/$LINUX_SOURCES_TARBALL" ]; then
	echo "Downloading the Linux sources tarball"
	wget -P "$SOURCES_DIR" $LINUX_MIRROR/$LINUX_SOURCES_TARBALL > "$LOG_PATH" 2>&1
	if [ $? -ne 0 ]
	then
		echo "Error: failed to download the Linux sources tarball."
		exit 1
	fi
fi

# get the full Aufs package name
AUFS_FULL_PACKAGE_NAME="aufs${AUFS_VERSION}-${LINUX_RELEASE_NUMBER}-git${BUILD_DATE}"

# download the Aufs sources through git
if [ ! -f "$SOURCES_DIR/${AUFS_FULL_PACKAGE_NAME}.tar.bz2" ]
then
	echo "Downloading the Aufs sources through git"
	git clone http://git.c3sl.ufpr.br/pub/scm/aufs/aufs2-standalone.git $AUFS_FULL_PACKAGE_NAME >> "$LOG_PATH" 2>&1
	if [ $? -ne 0 ]
	then
		echo "Error: failed to download the Aufs sources through git."
		exit 1
	fi
	cd $AUFS_FULL_PACKAGE_NAME
	git checkout origin/aufs${AUFS_VERSION}-${LINUX_RELEASE_NUMBER} >> "$LOG_PATH" 2>&1
	if [ $? -ne 0 ]
	then
		echo "Error: failed to download the Aufs sources through git."
		exit 1
	fi
	rm -rf .git
	cd ..
	echo "Creating the Aufs sources tarball"
	tar -c $AUFS_FULL_PACKAGE_NAME | bzip2 -9 > "$SOURCES_DIR/${AUFS_FULL_PACKAGE_NAME}.tar.bz2"
else
	echo "Extracting the Aufs sources tarball"
	tar xf "$SOURCES_DIR/${AUFS_FULL_PACKAGE_NAME}.tar.bz2" >> "$LOG_PATH" 2>&1
	if [ $? -ne 0 ]
	then
		echo "Error: failed to extract the Aufs sources tarball."
		exit 1
	fi
fi

# patch Aufs
echo "Patching the Aufs sources"
patch -d $AUFS_FULL_PACKAGE_NAME -p1 < aufs-allow-sfs.patch >> "$LOG_PATH" 2>&1
if [ $? -ne 0 ]
then
	echo "Error: failed to patch the Aufs sources."
	exit 1
fi
cp aufs-allow-sfs.patch "$PATCHES_DIR"

# extract the Linux sources tarball
echo "Extracting the Linux sources tarball"
tar xf "$SOURCES_DIR/$LINUX_SOURCES_TARBALL" >> "$LOG_PATH" 2>&1
if [ $? -ne 0 ]
then
	echo "Error: failed to extract the Linux sources tarball."
	exit 1
fi


cd linux-${LINUX_VERSION}

echo "Cleaning the Linux sources"
make clean >> "$LOG_PATH" 2>&1
make mrproper >> "$LOG_PATH" 2>&1

# get the full Linux API headers package name
LINUX_HEADERS_FULL_PACKAGE_NAME="${LINUX_HEADERS_PACKAGE_NAME}-${LINUX_MAJOR_VERSION}-${PACKAGE_NAME_SUFFIX}"

echo "Creating the Linux API headers package"
echo "  checking the headers"
make headers_check >> "$LOG_PATH" 2>&1
echo "  installing the headers"
make INSTALL_HDR_PATH=$LINUX_HEADERS_FULL_PACKAGE_NAME headers_install >> "$LOG_PATH" 2>&1
echo "  removing unneeded files"
find $LINUX_HEADERS_FULL_PACKAGE_NAME/include \( -name .install -o -name ..install.cmd \) -delete
mv $LINUX_HEADERS_FULL_PACKAGE_NAME "$PACKAGES_DIR"

# patch the Linux sources with the Aufs patches and add the file system code itself
echo "Adding Aufs to the Linux sources"
for i in kbuild base standalone; do
	patch -p1 < ../$AUFS_FULL_PACKAGE_NAME/aufs2-$i.patch >> "$LOG_PATH" 2>&1
	if [ $? -ne 0 ]
	then
		echo "Error: failed to add Aufs to the Linux sources."
		exit 1
	fi
done
cp -r ../$AUFS_FULL_PACKAGE_NAME/{fs,Documentation} .
cp -r ../$AUFS_FULL_PACKAGE_NAME/include/linux/aufs_type.h include/linux
cat ../$AUFS_FULL_PACKAGE_NAME/include/linux/Kbuild >> include/linux/Kbuild

echo "Resetting the minor version number"
cp Makefile Makefile-orig
sed -i s/'^EXTRAVERSION = .*$'/'EXTRAVERSION = '/ Makefile
echo "  creating a patch"
diff -up Makefile-orig Makefile > "$PATCHES_DIR/extra-version.patch"
rm Makefile-orig

echo "Reducing the number of consoles"
cp kernel/printk.c kernel/printk.c-orig
sed -i s/'#define MAX_CMDLINECONSOLES 8'/'#define MAX_CMDLINECONSOLES 5'/ kernel/printk.c
echo "  creating a patch"
diff -up kernel/printk.c-orig kernel/printk.c > "$PATCHES_DIR/less-consoles.patch"

echo "Reducing the verbosity level"
cp -f kernel/printk.c kernel/printk.c-orig
sed -i s/'#define DEFAULT_CONSOLE_LOGLEVEL 7 \/\* anything MORE serious than KERN_DEBUG \*\/'/'#define DEFAULT_CONSOLE_LOGLEVEL 3 \/\* anything MORE serious than KERN_ERR \*\/'/ kernel/printk.c
echo "  creating a patch"
diff -up kernel/printk.c-orig kernel/printk.c > "$PATCHES_DIR/lower-verbosity.patch"

echo "Applying patches"
for patch in ../patches/*
do
	echo "  applying `basename $patch`"
	patch -p1 < "$patch" >> "$LOG_PATH" 2>&1
	if [ $? -ne 0 ]
	then
		echo "Error: failed to apply $patch on the Linux sources."
		exit 1
	fi
	cp "$patch" "$PATCHES_DIR"
done

echo "Adding the Puppy Linux boot logo"
echo "  creating a clean copy of the sources"
cp -r drivers/video/logo drivers/video/logo-orig
echo "  applying the Puppy Linux logo patch"
patch -p1 < ../puppy-logo.patch >> "$LOG_PATH" 2>&1
if [ $? -ne 0 ]
then
	echo "Error: failed to apply the Puppy Linux logo patch on the Linux sources."
	exit 1
fi

# resize the image and make the background black	
resized_image=`mktemp -u`
pngtopnm -mix -background black "$LOGO_IMAGE" | pnmscale -width $LOGO_WIDTH -height $LOGO_HEIGHT > $resized_image
if [ $? -ne 0 ]
then
	echo "Error: failed to resize ${LOGO_IMAGE}."
	exit 1
fi

# create 224-color, 16-color and monochrome variants
echo "  creating 224-color, 16-color and monochrome variants"
cat $resized_image | ppmquant 224 | pnmnoraw > drivers/video/logo/logo_puppy_clut224.ppm
cat $resized_image | ppmquant 16 | pnmnoraw > drivers/video/logo/logo_puppy_vga16.ppm
cat $resized_image | ppmquant 2 | pnmnoraw > drivers/video/logo/logo_puppy_mono.ppm

# delete the resized image
rm -f $resized_image

cd ..
echo "  creating a patch"
diff -rupN linux-${LINUX_VERSION}/drivers/video/logo-orig linux-${LINUX_VERSION}/drivers/video/logo > "$PATCHES_DIR/puppy-logo.patch"
rm -rf linux-${LINUX_VERSION}/drivers/video/logo-orig
cd linux-${LINUX_VERSION}

echo "  cleaning up patch clutter"
find . -name '*.orig' -delete
find . -name '*.rej' -delete
find . -name '*~' -delete

echo "Building Linux"
cp ../DOTconfig .config
make bzImage modules >> "$LOG_PATH" 2>&1
if [[ ! -f arch/x86/boot/bzImage || ! -f System.map ]]; then
	echo "Error: failed to build Linux."
	exit 1
fi
cp .config "$SOURCES_DIR/DOTconfig-${LINUX_VERSION}-${BUILD_DATE}"

# get the full Linux image and modules package name
LINUX_FULL_PACKAGE_NAME="${LINUX_PACKAGE_NAME}-${LINUX_RELEASE_NUMBER}-${PACKAGE_NAME_SUFFIX}"

echo "Creating the Linux image and modules package"
echo "  installing the kernel modules"
make INSTALL_MOD_PATH=$LINUX_FULL_PACKAGE_NAME modules_install
echo "  removing links to the sources"
rm -f $LINUX_FULL_PACKAGE_NAME/lib/modules/$LINUX_MAJOR_VERSION/{build,source}
ln -s $LINUX_MAJOR_VERSION $LINUX_FULL_PACKAGE_NAME/lib/modules/$LINUX_VERSION
echo "  stripping modules"
for module in `find $LINUX_FULL_PACKAGE_NAME -name *.ko -type f`
do
	strip --strip-unneeded $module >> "$LOG_PATH" 2>&1
	if [ $? -ne 0 ]
	then
		echo "Error: failed to strip the Linux modules."
		exit 1
	fi
done
echo "  adding the kernel image and System.map"
mkdir $LINUX_FULL_PACKAGE_NAME/boot
cp `find arch -name bzImage -type f` $LINUX_FULL_PACKAGE_NAME/boot/vmlinuz
cp System.map $LINUX_FULL_PACKAGE_NAME/boot
mv $LINUX_FULL_PACKAGE_NAME "$PACKAGES_DIR"

echo "Cleaning the Linux sources"
make clean >> "$LOG_PATH" 2>&1
make prepare >> "$LOG_PATH" 2>&1

cd ..

# get the full Linux sources package name
LINUX_SOURCES_FULL_PACKAGE_NAME="${LINUX_SOURCES_PACKAGE_NAME}-${LINUX_RELEASE_NUMBER}-${PACKAGE_NAME_SUFFIX}"

echo "Creating a Linux sources SFS extension"
mkdir -p $LINUX_SOURCES_FULL_PACKAGE_NAME/usr/src
mv linux-${LINUX_VERSION} $LINUX_SOURCES_FULL_PACKAGE_NAME/usr/src/linux
mkdir -p $LINUX_SOURCES_FULL_PACKAGE_NAME/lib/modules/$LINUX_MAJOR_VERSION
echo "  creating required links"
ln -s /usr/src/linux $LINUX_SOURCES_FULL_PACKAGE_NAME/lib/modules/$LINUX_MAJOR_VERSION/build
ln -s /usr/src/linux $LINUX_SOURCES_FULL_PACKAGE_NAME/lib/modules/$LINUX_MAJOR_VERSION/source
mv $LINUX_SOURCES_FULL_PACKAGE_NAME "$PACKAGES_DIR"

echo "Compressing the log"
bzip2 -9 "$LOG_PATH" 
cp "${LOG_PATH}.bz2" "$SOURCES_DIR"

echo "Compressing the patches"
for patch in "$PATCHES_DIR"/*
do
	bzip2 -9 "$patch"
done

echo "Done!"
aplay /usr/share/sounds/2barks.au
